# frozen_string_literal: true

module Trefoil
  # Represents an array of Post objects with additional convenience methods
  class Thread
    def initialize(client, board, id)
      @client = client
      @board = board
      @id = id
      @posts = nil
    end

    # Get a post based on its position
    # @param index [Integer] the index of the desired post
    # @return [Post, nil] A post object or nil if out of bounds.
    def [](index)
      posts[index]
    end

    # An array of all posts in the thread
    # @return [Array<Post>] All posts in this thread since last update
    def posts
      fetch_posts if @posts.nil?
      @posts
    end

    # Convenience method to get the first post in a thread
    # @return [Post]
    def op
      posts[0]
    end

    # Refresh post list
    def update
      fetch_posts
    end

    private

    # Fetch all posts into
    def fetch_posts
      @posts = []
      @client.get("/#{@board.name}/thread/#{@id}.json")[:posts].each do |post|
        @posts << Post.new(@client, @board, self, post)
      end
    end
  end

  # Used for catalog
  class ThreadStub < Thread
    attr_reader :op

    def initialize(client, board, id, op)
      @op = op
      super(client, board, id)
    end
  end
end
